body {
    width:100vw;
    height:100vh;
    margin: 0;
    display:flex;
    flex-direction:column;
    align-items:center;
    background-color: var(--theme-bg);
    color: var(--theme-text-bright);
}
.header-bar {
    display:flex;
    justify-content: center;
    align-items:center;
    position: sticky;
    top: 0px;
    width:100%;
    margin-bottom: 10px;
}
.wrapper {
    margin: 50px;
    display: grid;
    grid-template-columns: repeat(3, 1fr);
    gap: 30px;
}
.monitor-block {
    width: 300px;
    height: 120px;
    background-color: var(--theme-table);
    border-radius: 5px;
    box-shadow: 2px 2px 4px var(--theme-table-shadow);
    display:flex;
    flex-direction:column;
    justify-content: center;
    align-items:center;
    cursor:pointer;
}
.monitor-block:hover {
    /* box-shadow: 10px 10px 5px var(--theme-table-shadow); */
    background-color: var(--theme-table-hover);
}

.block-title {
    flex: 2;
    display:flex;
    justify-content: center;
    align-items:center;
}
.block-msg {
    flex: 1;
    display:flex;
    flex-direction:row;
    justify-content: flex-end;
    align-items: center;
    width:100%;
    background-color: var(--theme-table-hover);
    font-size: 0.8rem;
}
.block-msg > div {
    display:flex;
    justify-content: center;
    align-items: center;
    padding: 2px;
    margin: 5px;
    width:25px;
    height:25px;
    border-radius: 50%;
    background-color: var(--theme-table);
}
